#!/bin/sh

gui="optimas_gui"
cmd="optimas"

## loading user's configuration informations
. ./config.in

## A FAIRE : verifier si g++, make, qmake et les bibli ont tous la meme architechture (64 ou 32 bits)
## try : sudo apt-get install qt qwt graphviz
## system architecture (x86_64 or iX86)
lib=lib
arch=`uname -m`
#[ "$arch" = "x86_64" ] && lib=lib64
#echo $lib ; read v
## checking OptiMAS sources directories.
[ ! -d "$cmd" ] && echo "$cmd sources directory not found !" && exit 1
[ ! -d "$gui" ] && echo "$gui sources directory not found !" && exit 1

## looking for make and g++
if [ ! "$(which make)" ]; then
	echo "ERROR: make not found !"
	echo "try to install it using your distribution package manager:"
	echo "e.g. sudo apt-get install make (on Debian/Ubuntu plateforms)"
	echo "or yum install make (on Redhat/Fedora/CentOS plateforms)"
	exit 1
fi

if [ ! "$(which g++)" ]; then
	echo "ERROR: g++ not found !"
	echo "try to install it using your distribution package manager:"
	echo "e.g. apt-get install g++ (on Debian/Ubuntu plateforms)"
	echo "or yum install gcc-c++ (on Redhat/Fedora/CentOS plateforms)"
	exit 1
fi
# g++ --version | head -1 | grep -Eo " [0-9]\.[0-9]\.[0-9]( |$)"

echo

isroot=
## OptiMAS files location:
if [ "$(id -u)" -eq 0 ]	# install as super-user.
then
	isroot="[ \"\$(id -u)\" -ne 0 ] && echo \"Error: this script must be launched as root !\" && exit 1"
	optimas_dir=/usr/share/OptiMAS
	bin_dir=/usr/local/bin #/usr/bin ?
	example_dir=/usr/share/OptiMAS/output
	doc_dir=/usr/share/OptiMAS/doc
	website_dir=/usr/share/OptiMAS/website
	#images_dir=/usr/share/OptiMAS/images
	
else					# install as common user.
	bin_dir=$HOME/bin
	[ ! -e "$bin_dir" ] && mkdir $bin_dir
	[ -e "$bin_dir" -a ! -d "$bin_dir" ] && echo "ERROR: $bin_dir already exists but is not a directory !" && exit 1

	optimas_dir=$HOME/.OptiMAS
	example_dir=$HOME/.OptiMAS/output
	doc_dir=$HOME/.OptiMAS/doc
	website_dir=$HOME/.OptiMAS/website
	#images_dir=$HOME/.OptiMAS/images
	
fi
#echo "optimas_dir : $optimas_dir ; bin_dir : $bin_dir"; echo "isroot : $isroot FIN" ; read v

## create the uninstall script.
uninstallsh="uninstall_optimas.sh"
cat >$bin_dir/$uninstallsh <<FIN
#!/bin/sh
# this script has been generated by $0

$isroot
rm -rf $optimas_dir 2>/dev/null
rm -f $bin_dir/$cmd 2>/dev/null
rm -f $bin_dir/$gui 2>/dev/null
rm -f $bin_dir/$uninstallsh 2>/dev/null

exit 0

FIN
chmod a+x $bin_dir/$uninstallsh

#echo "uninstall script done !"; read v

[ ! -d "$optimas_dir" ] && mkdir -p $optimas_dir
## copy doc output website to optimas_dir
cd $gui
cp -rf input doc output website $optimas_dir
chmod -R a+r $optimas_dir
find $optimas_dir -type d -exec chmod a+x {} \;
cd ..
echo "optimas_dir: " $optimas_dir
echo "bin_dir: " $bin_dir
echo

############## optimas stuff (OptiMAS heavy treatment) ####################

cd $cmd
echo "Building $cmd ..."
make all
[ "$?" -ne 0 ] && echo "ERROR: make failed to build $cmd !" && exit 1

# copy optimas binary file to the bin directory
cp -f bin/$cmd $bin_dir
[ "$?" -ne 0 ] && echo "ERROR: could not copy $cmd in $bin_dir" && exit 1
#cp -rf ../$gui/input $optimas_dir
########################################################################

echo
cd ..

#echo "$cmd done"; read v

if [ "$1" = "--no-gui" ]	# don't install OptiMAS GUI.
then
	echo "$cmd is installed here: $bin_dir/$cmd"
	echo
else						# install OptiMAS GUI.
	
	echo
	echo "Note: This package comes with no Graphical User Interface"
	echo "$cmd is installed here: $bin_dir/$cmd"
	echo

fi # end of if [ "$1" = "--no-gui" ]
#echo ""; read v
## checking if bin_dir is in the PATH and/or user profile file.
echo $PATH | grep -q "$bin_dir"
if [ "$?" -ne 0 ] # bin_dir is not in the PATH
then
	echo "$bin_dir is not in the PATH environment variable"
	profile=
	if [ -f "$HOME/.${SHELL}_profile" ]; then
		profile="$HOME/.${SHELL}_profile"
	elif [ -f "$HOME/.${SHELL}_login" ]; then
		profile="$HOME/.${SHELL}_login"
	elif [ -f "$HOME/.profile" ]; then
		profile="$HOME/.profile"
	fi
	
	if [ -n "$profile" ] # user profile exists but $bin_dir is not in the PATH
	then
		. $profile
		echo $PATH | grep -q "$bin_dir"
		if [ "$?" -ne 0 ]  # do not update $profile once more in case of successive execution of this script.
		then
			#tmp="${bin_dir/$HOME/\$HOME}" # <= bashism !
			tmp=$(echo "$bin_dir" | perl -ne 's/$ENV{"HOME"}/\$HOME/g ; chomp; print')
			#echo "you may want to edit $profile and add the following lines:"
			echo >> $profile
			echo "if [ -d \"$tmp\" ] ; then" >> $profile
			echo "	PATH=\"\$PATH:$tmp\"" >> $profile
			echo "fi" >> $profile
		fi
	else # user profile does not exists
		cp -f ./user_profile $HOME/.profile
		profile="$HOME/.profile"
	fi
	echo -ne "Restart your session or type:\n. $profile\nto include $bin_dir in the PATH variable.\n"
	echo
	echo -ne "Then "
fi
#echo ""; read v
echo -ne "To launch $cmd, just type from the commande line:\n\
$cmd [arguments]\n\n\
See README file and $website_dir/index.html for OptiMAS documentation.\n\n"

echo "To uninstall OptiMAS, type the command:"
echo "$bin_dir/$uninstallsh"
echo

exit 0
